@@ -137,7 +137,7 @@ def usage():
     print(" update     update pegtree (via pip)")
 
 
-showingTPEG = True
+showingTPEG = False
 
 
 def load_grammar(options, default=None):
@@ -262,8 +262,12 @@ def test(options):
         with open(file) as f:
             for line in f:
                 lines += 1
-                t = parser(line)
-                fail += dumpError(lines, line, t)
+                try:
+                    t = parser(line)
+                    fail += dumpError(lines, line, t)
+                except RecursionError:
+                    print(color('Red', line))
+                    fail += 1
     et = time.time()
     if lines > 0:
         print(f'{fail}/{lines} {fail/lines} {(et - st) * 1000.0} ms')
